@@ -1,8 +1,5 @@
-# Allow conans to import ConanFile from here
-# to allow refactors
 from conans.client.build.autotools_environment import AutoToolsBuildEnvironment
 from conans.client.build.cmake import CMake
-from conans.client.toolchain.cmake import CMakeToolchain
 from conans.client.build.meson import Meson
 from conans.client.build.msbuild import MSBuild
 from conans.client.build.visual_environment import VisualStudioBuildEnvironment
@@ -12,6 +9,7 @@ from conans.model.options import Options
 from conans.model.settings import Settings
 from conans.util.files import load
 
+
 # complex_search: With ORs and not filtering by not restricted settings
 COMPLEX_SEARCH_CAPABILITY = "complex_search"
 CHECKSUM_DEPLOY = "checksum_deploy"  # Only when v2
@@ -22,4 +20,4 @@ OAUTH_TOKEN = "oauth_token"
 SERVER_CAPABILITIES = [COMPLEX_SEARCH_CAPABILITY, REVISIONS]  # Server is always with revisions
 DEFAULT_REVISION_V1 = "0"
 
-__version__ = '1.28.0'
+__version__ = '1.43.2'
