@@ -28,18 +28,15 @@ class _RecipeBuildRequires(OrderedDict):
         for build_require in build_requires:
             self.add(build_require, context=self._default_context)
 
-    def add(self, build_require, context):
+    def add(self, build_require, context, force_host_context=False):
         if not isinstance(build_require, ConanFileReference):
             build_require = ConanFileReference.loads(build_require)
+        build_require.force_host_context = force_host_context  # Dirty, but will be removed in 2.0
         self[(build_require.name, context)] = build_require
 
     def __call__(self, build_require, force_host_context=False):
         context = CONTEXT_HOST if force_host_context else self._default_context
-        self.add(build_require, context)
-
-    def update(self, build_requires):
-        for build_require in build_requires:
-            self.add(build_require)
+        self.add(build_require, context, force_host_context)
 
     def __str__(self):
         items = ["{} ({})".format(br, ctxt) for (_, ctxt), br in self.items()]
@@ -88,6 +85,8 @@ class GraphManager(object):
             if graph_lock and not test:  # Only lock python requires if it is not test_package
                 node_id = graph_lock.get_consumer(graph_info.root)
                 lock_python_requires = graph_lock.python_requires(node_id)
+            # The global.conf is necessary for download_cache definition
+            profile_host.conf.rebase_conf_definition(self._cache.new_config)
             conanfile = self._loader.load_consumer(conanfile_path,
                                                    profile_host=profile_host,
                                                    name=name, version=version,
@@ -101,7 +100,6 @@ class GraphManager(object):
             if test:
                 conanfile.display_name = "%s (test package)" % str(test)
                 conanfile.output.scope = conanfile.display_name
-
             run_configure_method(conanfile, down_options=None, down_ref=None, ref=None)
         else:
             conanfile = self._loader.load_conanfile_txt(conanfile_path, profile_host=profile_host)
@@ -111,47 +109,63 @@ class GraphManager(object):
         return conanfile
 
     def load_graph(self, reference, create_reference, graph_info, build_mode, check_updates, update,
-                   remotes, recorder, apply_build_requires=True):
+                   remotes, recorder, apply_build_requires=True, lockfile_node_id=None,
+                   is_build_require=False, require_overrides=None):
         """ main entry point to compute a full dependency graph
         """
-        root_node = self._load_root_node(reference, create_reference, graph_info)
-        deps_graph = self._resolve_graph(root_node, graph_info, build_mode, check_updates, update,
-                                         remotes, recorder,
-                                         apply_build_requires=apply_build_requires)
+        profile_host, profile_build = graph_info.profile_host, graph_info.profile_build
+        graph_lock, root_ref = graph_info.graph_lock, graph_info.root
 
+        root_node = self._load_root_node(reference, create_reference, profile_host, graph_lock,
+                                         root_ref, lockfile_node_id, is_build_require,
+                                         require_overrides)
+        deps_graph = self._resolve_graph(root_node, profile_host, profile_build, graph_lock,
+                                         build_mode, check_updates, update, remotes, recorder,
+                                         apply_build_requires=apply_build_requires)
         # Run some validations once the graph is built
         self._validate_graph_provides(deps_graph)
 
+        # THIS IS NECESSARY to store dependencies options in profile, for consumer
+        # FIXME: This is a hack. Might dissapear if graph for local commands is always recomputed
+        graph_info.options = root_node.conanfile.options.values
+        if root_node.ref:
+            graph_info.root = root_node.ref
+
+        if graph_info.graph_lock is None:
+            graph_info.graph_lock = GraphLock(deps_graph, self._cache.config.revisions_enabled)
+
         return deps_graph
 
-    def _load_root_node(self, reference, create_reference, graph_info):
+    def _load_root_node(self, reference, create_reference, profile_host, graph_lock, root_ref,
+                        lockfile_node_id, is_build_require, require_overrides):
         """ creates the first, root node of the graph, loading or creating a conanfile
         and initializing it (settings, options) as necessary. Also locking with lockfile
         information
         """
-        profile = graph_info.profile_host
-        graph_lock = graph_info.graph_lock
-        profile.dev_reference = create_reference  # Make sure the created one has develop=True
+        profile_host.dev_reference = create_reference  # Make sure the created one has develop=True
 
         if isinstance(reference, list):  # Install workspace with multiple root nodes
-            conanfile = self._loader.load_virtual(reference, profile, scope_options=False)
+            conanfile = self._loader.load_virtual(reference, profile_host, scope_options=False)
             # Locking in workspaces not implemented yet
             return Node(ref=None, context=CONTEXT_HOST, conanfile=conanfile, recipe=RECIPE_VIRTUAL)
 
         # create (without test_package), install|info|graph|export-pkg <ref>
         if isinstance(reference, ConanFileReference):
-            return self._load_root_direct_reference(reference, graph_lock, profile)
+            return self._load_root_direct_reference(reference, graph_lock, profile_host,
+                                                    lockfile_node_id, is_build_require,
+                                                    require_overrides)
 
         path = reference  # The reference must be pointing to a user space conanfile
         if create_reference:  # Test_package -> tested reference
-            return self._load_root_test_package(path, create_reference, graph_lock, profile)
+            return self._load_root_test_package(path, create_reference, graph_lock, profile_host,
+                                                require_overrides)
 
         # It is a path to conanfile.py or conanfile.txt
-        root_node, ref = self._load_root_consumer(path, graph_lock, profile, graph_info.root)
-        graph_info.root = ref  # To store it for later calls
+        root_node = self._load_root_consumer(path, graph_lock, profile_host, root_ref,
+                                             require_overrides)
         return root_node
 
-    def _load_root_consumer(self, path, graph_lock, profile, ref):
+    def _load_root_consumer(self, path, graph_lock, profile, ref, require_overrides):
         """ load a CONSUMER node from a user space conanfile.py or conanfile.txt
         install|info|create|graph <path>
         :path full path to a conanfile
@@ -166,7 +180,9 @@ class GraphManager(object):
                 if ref.name is None:
                     # If the graph_info information is not there, better get what we can from
                     # the conanfile
-                    conanfile = self._loader.load_basic(path)
+                    # Using load_named() to run set_name() set_version() and get them
+                    # so it can be found by name in the lockfile
+                    conanfile = self._loader.load_named(path, None, None, None, None)
                     ref = ConanFileReference(ref.name or conanfile.name,
                                              ref.version or conanfile.version,
                                              ref.user, ref.channel, validate=False)
@@ -178,13 +194,15 @@ class GraphManager(object):
                                                    version=ref.version,
                                                    user=ref.user,
                                                    channel=ref.channel,
-                                                   lock_python_requires=lock_python_requires)
+                                                   lock_python_requires=lock_python_requires,
+                                                   require_overrides=require_overrides)
 
             ref = ConanFileReference(conanfile.name, conanfile.version,
                                      ref.user, ref.channel, validate=False)
             root_node = Node(ref, conanfile, context=CONTEXT_HOST, recipe=RECIPE_CONSUMER, path=path)
         else:
-            conanfile = self._loader.load_conanfile_txt(path, profile, ref=ref)
+            conanfile = self._loader.load_conanfile_txt(path, profile, ref=ref,
+                                                        require_overrides=require_overrides)
             root_node = Node(None, conanfile, context=CONTEXT_HOST, recipe=RECIPE_CONSUMER,
                              path=path)
 
@@ -192,9 +210,10 @@ class GraphManager(object):
             node_id = graph_lock.get_consumer(root_node.ref)
             root_node.id = node_id
 
-        return root_node, ref
+        return root_node
 
-    def _load_root_direct_reference(self, reference, graph_lock, profile):
+    def _load_root_direct_reference(self, reference, graph_lock, profile, lockfile_node_id,
+                                    is_build_require, require_overrides):
         """ When a full reference is provided:
         install|info|graph <ref> or export-pkg .
         :return a VIRTUAL root_node with a conanfile that requires the reference
@@ -203,13 +222,17 @@ class GraphManager(object):
             raise ConanException("Revisions not enabled in the client, specify a "
                                  "reference without revision")
 
-        conanfile = self._loader.load_virtual([reference], profile)
+        conanfile = self._loader.load_virtual([reference], profile,
+                                              is_build_require=is_build_require,
+                                              require_overrides=require_overrides)
         root_node = Node(ref=None, conanfile=conanfile, context=CONTEXT_HOST, recipe=RECIPE_VIRTUAL)
-        if graph_lock:  # Find the Node ID in the lock of current root
-            graph_lock.find_require_and_lock(reference, conanfile)
+        # Build_requires cannot be found as early as this, because there is no require yet
+        if graph_lock and not is_build_require:  # Find the Node ID in the lock of current root
+            graph_lock.find_require_and_lock(reference, conanfile, lockfile_node_id)
         return root_node
 
-    def _load_root_test_package(self, path, create_reference, graph_lock, profile):
+    def _load_root_test_package(self, path, create_reference, graph_lock, profile,
+                                require_overrides):
         """ when a test_package/conanfile.py is provided, together with the reference that is
         being created and need to be tested
         :return a CONSUMER root_node with a conanfile.py with an injected requires to the
@@ -218,15 +241,31 @@ class GraphManager(object):
         test = str(create_reference)
         # do not try apply lock_python_requires for test_package/conanfile.py consumer
         conanfile = self._loader.load_consumer(path, profile, user=create_reference.user,
-                                               channel=create_reference.channel)
+                                               channel=create_reference.channel,
+                                               require_overrides=require_overrides
+                                               )
         conanfile.display_name = "%s (test package)" % str(test)
         conanfile.output.scope = conanfile.display_name
-        # Injecting the tested reference
-        require = conanfile.requires.get(create_reference.name)
-        if require:
-            require.ref = require.range_ref = create_reference
-        else:
-            conanfile.requires.add_ref(create_reference)
+
+        # Injection of the tested reference
+        test_type = getattr(conanfile, "test_type", ("requires", ))
+        if not isinstance(test_type, (list, tuple)):
+            test_type = (test_type, )
+        if "build_requires" in test_type:
+            if getattr(conanfile, "build_requires", None):
+                # Injecting the tested reference
+                existing = conanfile.build_requires
+                if not isinstance(existing, (list, tuple)):
+                    existing = [existing]
+                conanfile.build_requires = list(existing) + [create_reference]
+            else:
+                conanfile.build_requires = str(create_reference)
+        if "requires" in test_type:
+            require = conanfile.requires.get(create_reference.name)
+            if require:
+                require.ref = require.range_ref = create_reference
+            else:
+                conanfile.requires.add_ref(create_reference)
         ref = ConanFileReference(conanfile.name, conanfile.version,
                                  create_reference.user, create_reference.channel, validate=False)
         root_node = Node(ref, conanfile, recipe=RECIPE_CONSUMER, context=CONTEXT_HOST, path=path)
@@ -234,28 +273,17 @@ class GraphManager(object):
             graph_lock.find_require_and_lock(create_reference, conanfile)
         return root_node
 
-    def _resolve_graph(self, root_node, graph_info, build_mode, check_updates,
-                       update, remotes, recorder, apply_build_requires=True):
+    def _resolve_graph(self, root_node, profile_host, profile_build, graph_lock, build_mode,
+                       check_updates, update, remotes, recorder, apply_build_requires=True):
         build_mode = BuildMode(build_mode, self._output)
-        profile_host = graph_info.profile_host
-        graph_lock = graph_info.graph_lock
         deps_graph = self._load_graph(root_node, check_updates, update,
                                       build_mode=build_mode, remotes=remotes,
                                       recorder=recorder,
                                       profile_host=profile_host,
-                                      profile_build=graph_info.profile_build,
+                                      profile_build=profile_build,
                                       apply_build_requires=apply_build_requires,
                                       graph_lock=graph_lock)
 
-        # THIS IS NECESSARY to store dependencies options in profile, for consumer
-        # FIXME: This is a hack. Might dissapear if graph for local commands is always recomputed
-        graph_info.options = root_node.conanfile.options.values
-        if root_node.ref:
-            graph_info.root = root_node.ref
-
-        if graph_info.graph_lock is None:
-            graph_info.graph_lock = GraphLock(deps_graph, self._cache.config.revisions_enabled)
-
         version_ranges_output = self._resolver.output
         if version_ranges_output:
             self._output.success("Version ranges solved")
@@ -270,6 +298,15 @@ class GraphManager(object):
     @staticmethod
     def _get_recipe_build_requires(conanfile, default_context):
         conanfile.build_requires = _RecipeBuildRequires(conanfile, default_context)
+
+        class TestRequirements:
+            def __init__(self, build_requires):
+                self._build_requires = build_requires
+
+            def __call__(self, ref):
+                self._build_requires(ref, force_host_context=True)
+
+        conanfile.test_requires = TestRequirements(conanfile.build_requires)
         if hasattr(conanfile, "build_requirements"):
             with get_env_context_manager(conanfile):
                 with conanfile_exception_formatter(str(conanfile), "build_requirements"):
@@ -290,21 +327,22 @@ class GraphManager(object):
             return
 
         for node in graph.ordered_iterate(nodes_subset):
-            # Virtual conanfiles doesn't have output, but conanfile.py and conanfile.txt do
-            # FIXME: To be improved and build a explicit model for this
-            if node.recipe == RECIPE_VIRTUAL:
-                continue
             # Packages with PACKAGE_ID_UNKNOWN might be built in the future, need build requires
             if (node.binary not in (BINARY_BUILD, BINARY_EDITABLE, BINARY_UNKNOWN)
-                and node.recipe != RECIPE_CONSUMER):
+                    and node.recipe not in (RECIPE_CONSUMER, RECIPE_VIRTUAL)):
                 continue
             package_build_requires = self._get_recipe_build_requires(node.conanfile, default_context)
             str_ref = str(node.ref)
+
+            #  Compute the update of the current recipe build_requires when updated with the
+            # downstream profile-defined build-requires
             new_profile_build_requires = []
             for pattern, build_requires in profile_build_requires.items():
+                if node.recipe == RECIPE_VIRTUAL:  # Virtual do not get profile build requires
+                    continue
                 if ((node.recipe == RECIPE_CONSUMER and pattern == "&") or
-                    (node.recipe != RECIPE_CONSUMER and pattern == "&!") or
-                    fnmatch.fnmatch(str_ref, pattern)):
+                        (node.recipe != RECIPE_CONSUMER and pattern == "&!") or
+                        fnmatch.fnmatch(str_ref, pattern)):
                     for build_require in build_requires:
                         br_key = (build_require.name, default_context)
                         if br_key in package_build_requires:  # Override defined
@@ -316,30 +354,37 @@ class GraphManager(object):
                         elif build_require.name != node.name or default_context != node.context:
                             new_profile_build_requires.append((build_require, default_context))
 
-            if package_build_requires:
-                br_list = [(it, ctxt) for (_, ctxt), it in package_build_requires.items()]
-                nodessub = builder.extend_build_requires(graph, node,
-                                                         br_list,
-                                                         check_updates, update, remotes,
-                                                         profile_host, profile_build, graph_lock)
-                build_requires = profile_build.build_requires if default_context == CONTEXT_BUILD \
-                    else profile_build_requires
-                self._recurse_build_requires(graph, builder,
-                                             check_updates, update, build_mode,
-                                             remotes, build_requires, recorder,
+            def _recurse_build_requires(br_list, transitive_build_requires):
+                nodessub = builder.extend_build_requires(graph, node, br_list, check_updates,
+                                                         update, remotes, profile_host,
+                                                         profile_build, graph_lock)
+                self._recurse_build_requires(graph, builder, check_updates, update, build_mode,
+                                             remotes, transitive_build_requires, recorder,
                                              profile_host, profile_build, graph_lock,
                                              nodes_subset=nodessub, root=node)
 
+            if package_build_requires:
+                if default_context == CONTEXT_BUILD:
+                    br_build, br_host = [], []
+                    for (_, ctxt), it in package_build_requires.items():
+                        if ctxt == CONTEXT_BUILD:
+                            br_build.append((it, ctxt))
+                        else:
+                            br_host.append((it, ctxt))
+                    if br_build:
+                        _recurse_build_requires(br_build, profile_build.build_requires)
+                    if br_host:
+                        _recurse_build_requires(br_host, profile_build_requires)
+                else:
+                    br_all = [(it, ctxt) for (_, ctxt), it in package_build_requires.items()]
+                    _recurse_build_requires(br_all, profile_build_requires)
+
             if new_profile_build_requires:
-                nodessub = builder.extend_build_requires(graph, node, new_profile_build_requires,
-                                                         check_updates, update, remotes,
-                                                         profile_host, profile_build, graph_lock)
+                _recurse_build_requires(new_profile_build_requires, {})
 
-                self._recurse_build_requires(graph, builder,
-                                             check_updates, update, build_mode,
-                                             remotes, {}, recorder,
-                                             profile_host, profile_build, graph_lock,
-                                             nodes_subset=nodessub, root=node)
+            if graph_lock and node.recipe != RECIPE_VIRTUAL:
+                graph_lock.check_locked_build_requires(node, package_build_requires,
+                                                       new_profile_build_requires)
 
     def _load_graph(self, root_node, check_updates, update, build_mode, remotes,
                     recorder, profile_host, profile_build, apply_build_requires,
@@ -403,10 +448,13 @@ def load_deps_info(current_path, conanfile, required):
         return
     info_file_path = os.path.join(current_path, BUILD_INFO)
     try:
-        deps_cpp_info, deps_user_info, deps_env_info = TXTGenerator.loads(load(info_file_path))
+        deps_cpp_info, deps_user_info, deps_env_info, user_info_build = \
+            TXTGenerator.loads(load(info_file_path), filter_empty=True)
         conanfile.deps_cpp_info = deps_cpp_info
         conanfile.deps_user_info = deps_user_info
         conanfile.deps_env_info = deps_env_info
+        if user_info_build:
+            conanfile.user_info_build = user_info_build
     except IOError:
         if required:
             raise ConanException("%s file not found in %s\nIt is required for this command\n"
