@@ -3,7 +3,7 @@ from collections import OrderedDict
 from copy import copy
 
 from conans.errors import ConanException
-from conans.util.conan_v2_mode import conan_v2_behavior
+from conans.util.conan_v2_mode import conan_v2_error
 
 DEFAULT_INCLUDE = "include"
 DEFAULT_LIB = "lib"
@@ -35,6 +35,67 @@ class DefaultOrderedDict(OrderedDict):
         return the_copy
 
 
+class BuildModulesDict(dict):
+    """
+    A dictionary with append and extend for cmake build modules to keep it backwards compatible
+    with the list interface
+    """
+
+    def __getitem__(self, key):
+        if key not in self.keys():
+            super(BuildModulesDict, self).__setitem__(key, list())
+        return super(BuildModulesDict, self).__getitem__(key)
+
+    def _append(self, item):
+        if item.endswith(".cmake"):
+            self["cmake"].append(item)
+            self["cmake_multi"].append(item)
+            self["cmake_find_package"].append(item)
+            self["cmake_find_package_multi"].append(item)
+
+    def append(self, item):
+        conan_v2_error("Use 'self.cpp_info.build_modules[\"<generator>\"].append(\"{item}\")' "
+                       'instead'.format(item=item))
+        self._append(item)
+
+    def extend(self, items):
+        conan_v2_error("Use 'self.cpp_info.build_modules[\"<generator>\"].extend({items})' "
+                       "instead".format(items=items))
+        for item in items:
+            self._append(item)
+
+    @classmethod
+    def from_list(cls, build_modules):
+        the_dict = BuildModulesDict()
+        the_dict.extend(build_modules)
+        return the_dict
+
+
+def dict_to_abs_paths(the_dict, rootpath):
+    new_dict = {}
+    for generator, values in the_dict.items():
+        new_dict[generator] = [os.path.join(rootpath, p) if not os.path.isabs(p) else p
+                               for p in values]
+    return new_dict
+
+
+def merge_lists(seq1, seq2):
+    return seq1 + [s for s in seq2 if s not in seq1]
+
+
+def merge_dicts(d1, d2):
+    def merge_lists(seq1, seq2):
+        return [s for s in seq1 if s not in seq2] + seq2
+
+    result = d1.copy()
+    for k, v in d2.items():
+        if k not in d1.keys():
+            result[k] = v
+        else:
+            result[k] = merge_lists(d1[k], d2[k])
+    return result
+
+
 class _CppInfo(object):
     """ Object that stores all the necessary information to build in C/C++.
     It is intended to be system independent, translation to
@@ -43,6 +104,7 @@ class _CppInfo(object):
 
     def __init__(self):
         self._name = None
+        self._generator_properties = {}
         self.names = {}
         self.system_libs = []  # Ordered list of system libraries
         self.includedirs = []  # Ordered list of include paths
@@ -60,12 +122,14 @@ class _CppInfo(object):
         self.cxxflags = []  # C++ compilation flags
         self.sharedlinkflags = []  # linker flags
         self.exelinkflags = []  # linker flags
-        self.build_modules = []
+        self.objects = []  # objects to link
+        self.build_modules = BuildModulesDict()  # FIXME: This should be just a plain dict
         self.filenames = {}  # name of filename to create for various generators
         self.rootpath = ""
         self.sysroot = ""
         self.requires = []
         self._build_modules_paths = None
+        self._build_modules = None
         self._include_paths = None
         self._lib_paths = None
         self._bin_paths = None
@@ -89,8 +153,14 @@ class _CppInfo(object):
     @property
     def build_modules_paths(self):
         if self._build_modules_paths is None:
-            self._build_modules_paths = [os.path.join(self.rootpath, p) if not os.path.isabs(p)
-                                         else p for p in self.build_modules]
+            if isinstance(self.build_modules, list):  # FIXME: This should be just a plain dict
+                conan_v2_error("Use 'self.cpp_info.build_modules[\"<generator>\"] = "
+                               "{the_list}' instead".format(the_list=self.build_modules))
+                self.build_modules = BuildModulesDict.from_list(self.build_modules)
+                # Invalidate necessary, get_build_modules used raise_incorrect_components_definition
+                self._build_modules = None
+            tmp = dict_to_abs_paths(BuildModulesDict(self.get_build_modules()), self.rootpath)
+            self._build_modules_paths = tmp
         return self._build_modules_paths
 
     @property
@@ -137,29 +207,49 @@ class _CppInfo(object):
 
     @property
     def name(self):
-        conan_v2_behavior("Use 'get_name(generator)' instead")
+        conan_v2_error("Use 'get_name(generator)' instead")
         return self._name
 
     @name.setter
     def name(self, value):
         self._name = value
 
-    def get_name(self, generator):
-        return self.names.get(generator, self._name)
+    # TODO: Deprecate for 2.0. Only cmake and pkg_config generators should access this.
+    #  Use get_property for 2.0
+    def get_name(self, generator, default_name=True):
+        property_name = None
+        if "pkg_config" in generator:
+            property_name = "pkg_config_name"
+        return self.get_property(property_name) \
+               or self.names.get(generator, self._name if default_name else None)
 
-    def get_filename(self, generator):
-        result = self.filenames.get(generator)
-        if result:
-            return result
-        return self.get_name(generator)
+    # TODO: Deprecate for 2.0. Only cmake generators should access this. Use get_property for 2.0
+    def get_filename(self, generator, default_name=True):
+        # Default to the legacy "names"
+        return self.filenames.get(generator) or self.names.get(generator, self._name if default_name else None)
+
+    # TODO: Deprecate for 2.0. Use get_property for 2.0
+    def get_build_modules(self):
+        if self._build_modules is None:  # Not cached yet
+            self._build_modules = self.build_modules
+        return self._build_modules
+
+    def set_property(self, property_name, value):
+        self._generator_properties[property_name] = value
+
+    def get_property(self, property_name):
+        try:
+            return self._generator_properties[property_name]
+        except KeyError:
+            pass
 
     # Compatibility for 'cppflags' (old style property to allow decoration)
     def get_cppflags(self):
-        conan_v2_behavior("'cpp_info.cppflags' is deprecated, use 'cxxflags' instead")
+        conan_v2_error("'cpp_info.cppflags' is deprecated, use 'cxxflags' instead")
         return self.cxxflags
 
     def set_cppflags(self, value):
-        conan_v2_behavior("'cpp_info.cppflags' is deprecated, use 'cxxflags' instead")
+        conan_v2_error("'cpp_info.cppflags' is deprecated, use 'cxxflags' instead")
         self.cxxflags = value
 
     cppflags = property(get_cppflags, set_cppflags)
@@ -167,19 +257,37 @@ class _CppInfo(object):
 
 class Component(_CppInfo):
 
-    def __init__(self, rootpath, version):
+    def __init__(self, rootpath, version, default_values):
         super(Component, self).__init__()
         self.rootpath = rootpath
-        self.includedirs.append(DEFAULT_INCLUDE)
-        self.libdirs.append(DEFAULT_LIB)
-        self.bindirs.append(DEFAULT_BIN)
-        self.resdirs.append(DEFAULT_RES)
-        self.builddirs.append(DEFAULT_BUILD)
-        self.frameworkdirs.append(DEFAULT_FRAMEWORK)
+        if default_values.includedir is not None:
+            self.includedirs.append(default_values.includedir)
+        if default_values.libdir is not None:
+            self.libdirs.append(default_values.libdir)
+        if default_values.bindir is not None:
+            self.bindirs.append(default_values.bindir)
+        if default_values.resdir is not None:
+            self.resdirs.append(default_values.resdir)
+        if default_values.builddir is not None:
+            self.builddirs.append(default_values.builddir)
+        if default_values.frameworkdir is not None:
+            self.frameworkdirs.append(default_values.frameworkdir)
         self.requires = []
         self.version = version
 
 
+class CppInfoDefaultValues(object):
+
+    def __init__(self, includedir=None, libdir=None, bindir=None,
+                 resdir=None, builddir=None, frameworkdir=None):
+        self.includedir = includedir
+        self.libdir = libdir
+        self.bindir = bindir
+        self.resdir = resdir
+        self.builddir = builddir
+        self.frameworkdir = frameworkdir
+
+
 class CppInfo(_CppInfo):
     """ Build Information declared to be used by the CONSUMERS of a
     conans. That means that consumers must use this flags and configs i order
@@ -187,18 +295,29 @@ class CppInfo(_CppInfo):
     Defined in user CONANFILE, directories are relative at user definition time
     """
 
-    def __init__(self, ref_name, root_folder):
+    def __init__(self, ref_name, root_folder, default_values=None):
         super(CppInfo, self).__init__()
         self._ref_name = ref_name
         self._name = ref_name
         self.rootpath = root_folder  # the full path of the package in which the conans is found
-        self.includedirs.append(DEFAULT_INCLUDE)
-        self.libdirs.append(DEFAULT_LIB)
-        self.bindirs.append(DEFAULT_BIN)
-        self.resdirs.append(DEFAULT_RES)
-        self.builddirs.append(DEFAULT_BUILD)
-        self.frameworkdirs.append(DEFAULT_FRAMEWORK)
-        self.components = DefaultOrderedDict(lambda: Component(self.rootpath, self.version))
+        self._default_values = default_values or CppInfoDefaultValues(DEFAULT_INCLUDE, DEFAULT_LIB,
+                                                                      DEFAULT_BIN, DEFAULT_RES,
+                                                                      DEFAULT_BUILD,
+                                                                      DEFAULT_FRAMEWORK)
+        if self._default_values.includedir is not None:
+            self.includedirs.append(self._default_values.includedir)
+        if self._default_values.libdir is not None:
+            self.libdirs.append(self._default_values.libdir)
+        if self._default_values.bindir is not None:
+            self.bindirs.append(self._default_values.bindir)
+        if self._default_values.resdir is not None:
+            self.resdirs.append(self._default_values.resdir)
+        if self._default_values.builddir is not None:
+            self.builddirs.append(self._default_values.builddir)
+        if self._default_values.frameworkdir is not None:
+            self.frameworkdirs.append(self._default_values.frameworkdir)
+        self.components = DefaultOrderedDict(lambda: Component(self.rootpath,
+                                                               self.version, self._default_values))
         # public_deps is needed to accumulate list of deps for cmake targets
         self.public_deps = []
         self._configs = {}
@@ -206,18 +325,18 @@ class CppInfo(_CppInfo):
     def __str__(self):
         return self._ref_name
 
-    def get_name(self, generator):
-        name = super(CppInfo, self).get_name(generator)
+    def get_name(self, generator, default_name=True):
+        name = super(CppInfo, self).get_name(generator, default_name=default_name)
 
         # Legacy logic for pkg_config generator
         from conans.client.generators.pkg_config import PkgConfigGenerator
         if generator == PkgConfigGenerator.name:
             fallback = self._name.lower() if self._name != self._ref_name else self._ref_name
             if PkgConfigGenerator.name not in self.names and self._name != self._name.lower():
-                conan_v2_behavior("Generated file and name for {gen} generator will change in"
-                                  " Conan v2 to '{name}'. Use 'self.cpp_info.names[\"{gen}\"]"
-                                  " = \"{fallback}\"' in your recipe to continue using current name."
-                                  .format(gen=PkgConfigGenerator.name, name=name, fallback=fallback))
+                conan_v2_error("Generated file and name for {gen} generator will change in"
+                               " Conan v2 to '{name}'. Use 'self.cpp_info.names[\"{gen}\"]"
+                               " = \"{fallback}\"' in your recipe to continue using current name."
+                               .format(gen=PkgConfigGenerator.name, name=name, fallback=fallback))
             name = self.names.get(generator, fallback)
         return name
 
@@ -231,12 +350,12 @@ class CppInfo(_CppInfo):
             result.filter_empty = self.filter_empty
             result.rootpath = self.rootpath
             result.sysroot = self.sysroot
-            result.includedirs.append(DEFAULT_INCLUDE)
-            result.libdirs.append(DEFAULT_LIB)
-            result.bindirs.append(DEFAULT_BIN)
-            result.resdirs.append(DEFAULT_RES)
-            result.builddirs.append(DEFAULT_BUILD)
-            result.frameworkdirs.append(DEFAULT_FRAMEWORK)
+            result.includedirs.append(self._default_values.includedir)
+            result.libdirs.append(self._default_values.libdir)
+            result.bindirs.append(self._default_values.bindir)
+            result.resdirs.append(self._default_values.resdir)
+            result.builddirs.append(self._default_values.builddir)
+            result.frameworkdirs.append(self._default_values.frameworkdir)
             return result
 
         return self._configs.setdefault(config, _get_cpp_info())
@@ -247,12 +366,18 @@ class CppInfo(_CppInfo):
 
         # Raise if mixing components
         if self.components and \
-            (self.includedirs != [DEFAULT_INCLUDE] or
-             self.libdirs != [DEFAULT_LIB] or
-             self.bindirs != [DEFAULT_BIN] or
-             self.resdirs != [DEFAULT_RES] or
-             self.builddirs != [DEFAULT_BUILD] or
-             self.frameworkdirs != [DEFAULT_FRAMEWORK] or
+            (self.includedirs != ([self._default_values.includedir]
+                if self._default_values.includedir is not None else []) or
+             self.libdirs != ([self._default_values.libdir]
+                if self._default_values.libdir is not None else []) or
+             self.bindirs != ([self._default_values.bindir]
+                if self._default_values.bindir is not None else []) or
+             self.resdirs != ([self._default_values.resdir]
+                if self._default_values.resdir is not None else []) or
+             self.builddirs != ([self._default_values.builddir]
+                if self._default_values.builddir is not None else []) or
+             self.frameworkdirs != ([self._default_values.frameworkdir]
+                if self._default_values.frameworkdir is not None  else []) or
              self.libs or
              self.system_libs or
              self.frameworks or
@@ -261,7 +386,8 @@ class CppInfo(_CppInfo):
              self.cxxflags or
              self.sharedlinkflags or
              self.exelinkflags or
-             self.build_modules or
+             self.objects or
+             self.get_build_modules() or
              self.requires):
             raise ConanException("self.cpp_info.components cannot be used with self.cpp_info "
                                  "global values at the same time")
@@ -286,7 +412,8 @@ class CppInfo(_CppInfo):
                 reason = None
                 if comp_require not in pkg_requires:
                     reason = "not defined as a recipe requirement"
-                elif package_requires[comp_require].private and package_requires[comp_require].override:
+                elif package_requires[comp_require].private and package_requires[
+                    comp_require].override:
                     reason = "it was defined as an overridden private recipe requirement"
                 elif package_requires[comp_require].private:
                     reason = "it was defined as a private recipe requirement"
@@ -333,7 +460,7 @@ class _BaseDepsCppInfo(_CppInfo):
         self.frameworkdirs = merge_lists(self.frameworkdirs, dep_cpp_info.framework_paths)
         self.libs = merge_lists(self.libs, dep_cpp_info.libs)
         self.frameworks = merge_lists(self.frameworks, dep_cpp_info.frameworks)
-        self.build_modules = merge_lists(self.build_modules, dep_cpp_info.build_modules_paths)
+        self.build_modules = merge_dicts(self.build_modules, dep_cpp_info.build_modules_paths)
         self.requires = merge_lists(self.requires, dep_cpp_info.requires)
         self.rootpaths.append(dep_cpp_info.rootpath)
 
@@ -343,7 +470,7 @@ class _BaseDepsCppInfo(_CppInfo):
         self.cflags = merge_lists(dep_cpp_info.cflags, self.cflags)
         self.sharedlinkflags = merge_lists(dep_cpp_info.sharedlinkflags, self.sharedlinkflags)
         self.exelinkflags = merge_lists(dep_cpp_info.exelinkflags, self.exelinkflags)
-
+        self.objects = merge_lists(dep_cpp_info.objects, self.objects)
         if not self.sysroot:
             self.sysroot = dep_cpp_info.sysroot
 
@@ -392,6 +519,7 @@ class DepCppInfo(object):
         self._cflags = None
         self._sharedlinkflags = None
         self._exelinkflags = None
+        self._objects = None
         self._requires = None
 
         self._include_paths = None
@@ -401,7 +529,7 @@ class DepCppInfo(object):
         self._res_paths = None
         self._src_paths = None
         self._framework_paths = None
-        self._build_module_paths = None
+        self._build_modules_paths = None
         self._sorted_components = None
         self._check_component_requires()
 
@@ -415,31 +543,31 @@ class DepCppInfo(object):
             attr = self._cpp_info.__getattr__(item)
         return attr
 
-    @staticmethod
-    def _merge_lists(seq1, seq2):
-        return seq1 + [s for s in seq2 if s not in seq1]
-
-    def _aggregated_values(self, item):
+    def _aggregated_dict_values(self, item):
         values = getattr(self, "_%s" % item)
         if values is not None:
             return values
-        values = getattr(self._cpp_info, item)
         if self._cpp_info.components:
+            values = {}
             for component in self._get_sorted_components().values():
-                values = self._merge_lists(values, getattr(component, item))
+                values = merge_dicts(values, getattr(component, item))
+        else:
+            values = getattr(self._cpp_info, item)
         setattr(self, "_%s" % item, values)
         return values
 
-    def _aggregated_paths(self, item):
-        paths = getattr(self, "_%s_paths" % item)
-        if paths is not None:
-            return paths
-        paths = getattr(self._cpp_info, "%s_paths" % item)
+    def _aggregated_list_values(self, item):
+        values = getattr(self, "_%s" % item)
+        if values is not None:
+            return values
         if self._cpp_info.components:
+            values = []
             for component in self._get_sorted_components().values():
-                paths = self._merge_lists(paths, getattr(component, "%s_paths" % item))
-        setattr(self, "_%s_paths" % item, paths)
-        return paths
+                values = merge_lists(values, getattr(component, item))
+        else:
+            values = getattr(self._cpp_info, item)
+        setattr(self, "_%s" % item, values)
+        return values
 
     @staticmethod
     def _filter_component_requires(requires):
@@ -488,71 +616,75 @@ class DepCppInfo(object):
 
     @property
     def build_modules_paths(self):
-        return self._aggregated_paths("build_modules")
+        return self._aggregated_dict_values("build_modules_paths")
 
     @property
     def include_paths(self):
-        return self._aggregated_paths("include")
+        return self._aggregated_list_values("include_paths")
 
     @property
     def lib_paths(self):
-        return self._aggregated_paths("lib")
+        return self._aggregated_list_values("lib_paths")
 
     @property
     def src_paths(self):
-        return self._aggregated_paths("src")
+        return self._aggregated_list_values("src_paths")
 
     @property
     def bin_paths(self):
-        return self._aggregated_paths("bin")
+        return self._aggregated_list_values("bin_paths")
 
     @property
     def build_paths(self):
-        return self._aggregated_paths("build")
+        return self._aggregated_list_values("build_paths")
 
     @property
     def res_paths(self):
-        return self._aggregated_paths("res")
+        return self._aggregated_list_values("res_paths")
 
     @property
     def framework_paths(self):
-        return self._aggregated_paths("framework")
+        return self._aggregated_list_values("framework_paths")
 
     @property
     def libs(self):
-        return self._aggregated_values("libs")
+        return self._aggregated_list_values("libs")
 
     @property
     def system_libs(self):
-        return self._aggregated_values("system_libs")
+        return self._aggregated_list_values("system_libs")
 
     @property
     def frameworks(self):
-        return self._aggregated_values("frameworks")
+        return self._aggregated_list_values("frameworks")
 
     @property
     def defines(self):
-        return self._aggregated_values("defines")
+        return self._aggregated_list_values("defines")
 
     @property
     def cxxflags(self):
-        return self._aggregated_values("cxxflags")
+        return self._aggregated_list_values("cxxflags")
 
     @property
     def cflags(self):
-        return self._aggregated_values("cflags")
+        return self._aggregated_list_values("cflags")
 
     @property
     def sharedlinkflags(self):
-        return self._aggregated_values("sharedlinkflags")
+        return self._aggregated_list_values("sharedlinkflags")
 
     @property
     def exelinkflags(self):
-        return self._aggregated_values("exelinkflags")
+        return self._aggregated_list_values("exelinkflags")
+
+    @property
+    def objects(self):
+        return self._aggregated_list_values("objects")
 
     @property
     def requires(self):
-        return self._aggregated_values("requires")
+        return self._aggregated_list_values("requires")
 
 
 class DepsCppInfo(_BaseDepsCppInfo):
